function results = SG(Y,mh,info)  
%% PURPOSE: Get the SG test
% --------------------------------------------------------
% USAGE: 
% where: Y  = Rx1 vector with quantitative data.
%        mh = Sxm matrix with m-sourrounding (use m_hist0.m)
%        info = an (optional) structure variable with input options
%         info.plot  = 'yes' / ['no'] flag for plotting symbols histogram
%         info.print = 'yes' / ['no'] print the result
% --------------------------------------------------------
%  RETURNS: results.sg - the value of the SG statistic
%           results.symb - the list of symbols
%           results.ns  - vector for producing bar plot with frequency of symbols
%           results.nusi- number of symbols
% --------------------------------------------------------
% References: 
% Lopez FA, Matilla M, Mur J and M Ruiz (2010) A Non-Parametric Spatial Independence Test
% Using Symbolic Entropy, Regional Science and Urban Economic. 40, 106-115
%
% Ruiz M, Lpez FA, A Pez. (2010) Testing for spatial association of qualitative 
% data using symbolic dynamics. Journal of Geographical Systems. 12 (3) 281-309
% --------------------------------------------------------
% SEE ALSO: m_hist0.m
% --------------------------------------------------------
%
% written by:
%
% Fernando A. Lopez Hernandez
% Facultad de C.C. de la Empresa
% Dpto. Metodos Cuantitativos e Informaticos
% Universidad Politecnica de Cartagena
% e-mail: fernando.lopez@upct.es

if nargin<3
    info.print='no';
    info.plot='no';
end
%%
[n1 m]=size(mh);
ZZ=zeros(n1,m);
ZZZ=zeros(n1,m-1);
S=zeros(1,n1);
%% Getting Symbols
symb=[0,1]';
for k=1:m-2,
    symb=[[ones(2^k,1);ones(2^k,1)-1],repmat(symb,2,1)];
end
results.symb=symb;
nusi=length(symb);
lns=zeros(1,nusi);

Z=(Y>=median(Y))*1;
ZZ=Z(mh);
%% Compare the central observation with those around him
ZZZ=ZZ(:,2:m)==repmat(ZZ(:,1),1,m-1);
%% 
S=zeros(n1,1);
for k=1:nusi
    S=S+ismember(ZZZ,symb(k,:),'rows')*k;
end
ns = hist(S',1:nusi);

for i=1:nusi
    if ns(i)==0 lns(i)=0; else lns(i)=log(ns(i)/n1); end
end
hm=-sum((ns/n1).*lns);
sg=2*n1*(log(nusi)-hm);
es95 = chi2inv(0.95,nusi-1);
results.sg=sg;
results.nusi=nusi;
results.ns=ns;
results.n1=n1;

if strcmp(info.plot,'yes')
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     Plot Symbols: Observed versus Expected
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
sf0=ones(length(ns),1)*(1/2)^(size(mh,2)-1)*n1;
    figure(1);
    %plot barsempirical frequency of symbols
    bar(results.ns/n1,'b');hold on
    %plot expected frequency of each symbol under null
    plot([1:nusi],sf0/n1,'.','markeredgecolor',[0.5 0.5 0.5],'markersize',5)
    %plot confidence intervals for expected frequency - NOTE: substitute 2
    %use expression for confidence interval
    for i=1:nusi
        line([i,i],...
            [sf0(i)/n1-norminv(.975,0,1)*sqrt((sf0(i)/n1*(1-sf0(i)/n1))/n1),...
            sf0(i)/n1+norminv(.975,0,1)*sqrt((sf0(i)/n1*(1-sf0(i)/n1))/n1)],...
            'color',[0.5 0.5 0.5],'marker','+','markersize',3);
    end
    set(gca,'xlim',[0,nusi+1]);
%    for i=1:nusi;XLs(i,:)=num2str(symb(i,1)*100+symb(i,2)*10+symb(i,3));end
    for i=1:nusi;XLs(i,:)=mat2str(symb(i,:));end
    set(gca,'XTickLabel',{XLs})    
    hold off
end

if strcmp(info.print,'yes')
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     Print Rresults
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
N=length(Y);
    fprintf(1,'\n')
    fprintf(1,'  Sample size               (N) = %9.0f\n',N)
    fprintf(1,'  Symbolized observations   (R) = %9.0f\n',n1)
    fprintf(1,'  Size of m-surrounding     (m) = %9.0f\n',m)
    fprintf(1,'  Degree of overlap         (r) = %9.0f\n',(m-(N-m)/(n1-1))+1)
    fprintf(1,'  Number of symbols         (n) = %9.0f\n',nusi)
    fprintf(1,'\n')
    fprintf(1,' SG Test for spatial dependence \n');
    fprintf(1,'  Test           Value     DF    p-value \n');
    fprintf(1,'   SG    =    %9.2f',sg);fprintf(1,'    %2.0f',nusi-1);fprintf(1,'   %9.4f \n',1-chi2cdf(sg,nusi-1));
end